@@ -822,7 +822,6 @@ real_exec_cmd (const char *cmd, gboolean in_terminal,
     gchar **free_envp = NULL;
     gchar **envp = environ;
     gchar **argv = NULL;
-    gboolean retval;
 
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
     SnLauncherContext *sn_context = NULL;
@@ -833,16 +832,16 @@ real_exec_cmd (const char *cmd, gboolean in_terminal,
     {
         if (in_terminal)
         {
-	    execute = g_strdup_printf ("xfterm4 %s", cmd);
+	    execute = g_strconcat ("xfterm4 ", cmd, NULL);
         }
         else
         {
-	    execute = g_strdup_printf ("xftree4 %s", cmd);
+	    execute = g_strconcat ("xftree4 ", cmd, NULL);
         }
     }
     else if (in_terminal)
     {
-	execute = g_strdup_printf ("xfterm4 -e %s", cmd);
+	execute = g_strconcat ("xfterm4 -e ", cmd, NULL);
     }
     else
     {
@@ -889,26 +888,26 @@ real_exec_cmd (const char *cmd, gboolean in_terminal,
 
     g_free (execute);
 
-    if (silent)
+    success =
+	g_spawn_async_with_pipes (NULL, argv, free_envp ? free_envp : envp,
+				  G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, 
+				  NULL, NULL, NULL, &error);
+    if (!success)
     {
-	retval =
-	    g_spawn_async (NULL, argv, free_envp ? free_envp : envp,
-			   G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error);
-	if (!retval)
+	if (error)
 	{
-	    if (error)
+	    if (silent)
 	    {
-		g_warning ("%s: %s\n", PACKAGE, error->message);
-		g_error_free (error);
+		g_warning ("%s", error->message);
 	    }
-	    success = FALSE;
+	    else
+	    {
+		xfce_err (error->message);
+	    }
+	    
+	    g_error_free (error);
 	}
     }
-    else
-    {
-	success =
-	    exec_command_full_with_envp (argv, free_envp ? free_envp : envp);
-    }
 
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
     if (use_sn)
